// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

mod adapter;
pub use self::adapter::Adapter;

mod aggregator;
pub use self::aggregator::Aggregator;

mod aggregator_pad;
pub use self::aggregator_pad::AggregatorPad;

mod base_parse;
pub use self::base_parse::BaseParse;

mod base_sink;
pub use self::base_sink::BaseSink;

mod base_src;
pub use self::base_src::BaseSrc;

mod base_transform;
pub use self::base_transform::BaseTransform;

mod push_src;
pub use self::push_src::PushSrc;

mod enums;
#[cfg(feature = "v1_18")]
#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
pub use self::enums::AggregatorStartTimeSelection;

mod flags;
pub use self::flags::BaseParseFrameFlags;

pub(crate) mod functions;

pub(crate) mod traits {
    pub use super::aggregator::AggregatorExt;
    pub use super::aggregator_pad::AggregatorPadExt;
    pub use super::base_parse::BaseParseExt;
    pub use super::base_sink::BaseSinkExt;
    pub use super::base_src::BaseSrcExt;
    pub use super::base_transform::BaseTransformExt;
}
